package cn.fateverse.query.mapper;

import cn.fateverse.query.entity.TableColumn;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @author Clay
 * @date 2023-07-16
 */
public interface TableColumnMapper {


    /**
     * 批量新增列信息
     *
     * @param columns 列信息
     * @return 处理结果
     */
    Integer batchInsert(List<TableColumn> columns);

    /**
     * 批量更新列信息
     *
     * @param columns 列信息
     * @return 处理结果
     */
    Integer batchUpdate(List<TableColumn> columns);

    /**
     * 同步时的批量更新
     *
     * @param columns 列信息
     * @return 处理结果
     */
    Integer syncUpdate(List<TableColumn> columns);

    /**
     * 批量删除列信息
     *
     * @param ids 需要删除的id
     * @return 影响数据条数
     */
    Integer batchRemove(@Param("ids") List<Long> ids);

    /**
     * 根据表格id删除列信息
     *
     * @param tableId 表格id
     * @return 删除行
     */
    Integer deleteByTableId(Long tableId);

    /**
     * 根据表格id查询列信息
     *
     * @param tableId 表格id
     * @return 列信息
     */
    List<TableColumn> selectByTableId(Long tableId);

    /**
     * 根据表格id获取列信息
     *
     * @param tableIds 表格id
     * @return 列信息
     */
    List<TableColumn> selectByTableIds(List<Long> tableIds);
}
